from __future__ import unicode_literals

import datetime
import itertools
import json
import logging
import multiprocessing
import os
import pprint
import re
import sys

from .avcodecs import video_codec_list, audio_codec_list, subtitle_codec_list
from .formats import format_list
from .ffmpeg import FFMpeg, FFMpegError, FFMpegConvertError

# Add pelican support
from pelican.generators import ArticlesGenerator
from pelican.generators import PagesGenerator
from pelican.settings import DEFAULT_CONFIG
from pelican import signals
from pelican.utils import pelican_open

logger = logging.getLogger(__name__)

class ConverterError(Exception):
    pass

class Converter(object):
    """
    Converter class, encapsulates formats and codecs.

    >>> c = Converter()
    """

    def __init__(self, ffmpeg_path=None, ffprobe_path=None):
        """
        Initialize a new Converter object.
        """

        self.ffmpeg = FFMpeg(ffmpeg_path=ffmpeg_path,
                             ffprobe_path=ffprobe_path)
        self.video_codecs = {}
        self.audio_codecs = {}
        self.subtitle_codecs = {}
        self.formats = {}

        for cls in audio_codec_list:
            name = cls.codec_name
            self.audio_codecs[name] = cls

        for cls in video_codec_list:
            name = cls.codec_name
            self.video_codecs[name] = cls

        for cls in subtitle_codec_list:
            name = cls.codec_name
            self.subtitle_codecs[name] = cls

        for cls in format_list:
            name = cls.format_name
            self.formats[name] = cls

    def parse_options(self, opt, twopass=None):
        """
        Parse format/codec options and prepare raw ffmpeg option list.
        """
        if not isinstance(opt, dict):
            raise ConverterError('Invalid output specification')

        if 'format' not in opt:
            raise ConverterError('Format not specified')

        f = opt['format']
        if f not in self.formats:
            raise ConverterError('Requested unknown format: ' + str(f))

        format_options = self.formats[f]().parse_options(opt)
        if format_options is None:
            raise ConverterError('Unknown container format error')

        if 'audio' not in opt and 'video' not in opt:
            raise ConverterError('Neither audio nor video streams requested')

        # audio options
        if 'audio' not in opt or twopass == 1:
            opt_audio = {'codec': None}
        else:
            opt_audio = opt['audio']
            if not isinstance(opt_audio, dict) or 'codec' not in opt_audio:
                raise ConverterError('Invalid audio codec specification')

        c = opt_audio['codec']
        if c not in self.audio_codecs:
            raise ConverterError('Requested unknown audio codec ' + str(c))

        audio_options = self.audio_codecs[c]().parse_options(opt_audio)
        if audio_options is None:
            raise ConverterError('Unknown audio codec error')

        # video options
        if 'video' not in opt:
            opt_video = {'codec': None}
        else:
            opt_video = opt['video']
            if not isinstance(opt_video, dict) or 'codec' not in opt_video:
                raise ConverterError('Invalid video codec specification')

        c = opt_video['codec']
        if c not in self.video_codecs:
            raise ConverterError('Requested unknown video codec ' + str(c))

        video_options = self.video_codecs[c]().parse_options(opt_video)
        if video_options is None:
            raise ConverterError('Unknown video codec error')

        if 'subtitle' not in opt:
            opt_subtitle = {'codec': None}
        else:
            opt_subtitle = opt['subtitle']
            if not isinstance(opt_subtitle, dict) or 'codec' not in opt_subtitle:
                raise ConverterError('Invalid subtitle codec specification')

        c = opt_subtitle['codec']
        if c not in self.subtitle_codecs:
            raise ConverterError('Requested unknown subtitle codec ' + str(c))

        subtitle_options = self.subtitle_codecs[c]().parse_options(opt_subtitle)
        if subtitle_options is None:
            raise ConverterError('Unknown subtitle codec error')

        if 'map' in opt:
            m = opt['map']
            if not type(m) == int:
                raise ConverterError('map needs to be int')
            else:
                format_options.extend(['-map', str(m)])


        # aggregate all options
        optlist = audio_options + video_options + subtitle_options + format_options

        if twopass == 1:
            optlist.extend(['-pass', '1'])
        elif twopass == 2:
            optlist.extend(['-pass', '2'])

        return optlist

    def convert(self, infile, outfile, options, twopass=False, timeout=10):
        """
        Convert media file (infile) according to specified options, and
        save it to outfile. For two-pass encoding, specify the pass (1 or 2)
        in the twopass parameter.

        Options should be passed as a dictionary. The keys are:
            * format (mandatory, string) - container format; see
              formats.BaseFormat for list of supported formats
            * audio (optional, dict) - audio codec and options; see
              avcodecs.AudioCodec for list of supported options
            * video (optional, dict) - video codec and options; see
              avcodecs.VideoCodec for list of supported options
            * map (optional, int) - can be used to map all content of stream 0

        Multiple audio/video streams are not supported. The output has to
        have at least an audio or a video stream (or both).

        Convert returns a generator that needs to be iterated to drive the
        conversion process. The generator will periodically yield timecode
        of currently processed part of the file (ie. at which second in the
        content is the conversion process currently).

        The optional timeout argument specifies how long should the operation
        be blocked in case ffmpeg gets stuck and doesn't report back. This
        doesn't limit the total conversion time, just the amount of time
        Converter will wait for each update from ffmpeg. As it's usually
        less than a second, the default of 10 is a reasonable default. To
        disable the timeout, set it to None. You may need to do this if
        using Converter in a threading environment, since the way the
        timeout is handled (using signals) has special restriction when
        using threads.

        >>> conv = Converter().convert('test1.ogg', '/tmp/output.mkv', {
        ...    'format': 'mkv',
        ...    'audio': { 'codec': 'aac' },
        ...    'video': { 'codec': 'h264' }
        ... })

        >>> for timecode in conv:
        ...   pass # can be used to inform the user about the progress
        """

        if not isinstance(options, dict):
            raise ConverterError('Invalid options')

        if not os.path.exists(infile):
            raise ConverterError("Source file doesn't exist: " + infile)

        info = self.ffmpeg.probe(infile)
        if info is None:
            raise ConverterError("Can't get information about source file")

        if not info.video and not info.audio:
            raise ConverterError('Source file has no audio or video streams')

        if info.video and 'video' in options:
            options = options.copy()
            v = options['video'] = options['video'].copy()
            v['src_width'] = info.video.video_width
            v['src_height'] = info.video.video_height

        if info.format.duration < 0.01:
            raise ConverterError('Zero-length media')

        if twopass:
            optlist1 = self.parse_options(options, 1)
            for timecode in self.ffmpeg.convert(infile, outfile, optlist1,
                                                timeout=timeout):
                yield int((50.0 * timecode) / info.format.duration)

            optlist2 = self.parse_options(options, 2)
            for timecode in self.ffmpeg.convert(infile, outfile, optlist2,
                                                timeout=timeout):
                yield int(50.0 + (50.0 * timecode) / info.format.duration)
        else:
            optlist = self.parse_options(options, twopass)
            for timecode in self.ffmpeg.convert(infile, outfile, optlist,
                                                timeout=timeout):
                yield int((100.0 * timecode) / info.format.duration)

    def probe(self, fname, posters_as_video=True):
        """
        Examine the media file. See the documentation of
        converter.FFMpeg.probe() for details.

        :param posters_as_video: Take poster images (mainly for audio files) as
            A video stream, defaults to True
        """
        return self.ffmpeg.probe(fname, posters_as_video)

    def thumbnail(self, fname, time, outfile, size=None, quality=FFMpeg.DEFAULT_JPEG_QUALITY):
        """
        Create a thumbnail of the media file. See the documentation of
        converter.FFMpeg.thumbnail() for details.
        """
        return self.ffmpeg.thumbnail(fname, time, outfile, size, quality)

    def thumbnails(self, fname, option_list):
        """
        Create one or more thumbnail of the media file. See the documentation
        of converter.FFMpeg.thumbnails() for details.
        """
        return self.ffmpeg.thumbnails(fname, option_list)

def initialized(pelican):
    p = os.path.expanduser('~/Videos')

    DEFAULT_CONFIG.setdefault('VIDEO_LIBRARY', p)
    DEFAULT_CONFIG.setdefault('VIDEO_GALLERY', (720, 400, 100))
    DEFAULT_CONFIG.setdefault('VIDEO_ARTICLE', (720, 400, 100))
    DEFAULT_CONFIG.setdefault('VIDEO_FORMAT', 'mp4')
    DEFAULT_CONFIG.setdefault('VIDEO_VCODEC', 'h264')
    DEFAULT_CONFIG.setdefault('VIDEO_ACODEC', 'aac')
    DEFAULT_CONFIG.setdefault('VIDEO_SAMPLERATE', 11025)
    DEFAULT_CONFIG.setdefault('VIDEO_CHANNELS', 2)
    DEFAULT_CONFIG.setdefault('VIDEO_CONVERT_JOBS', 1)
    DEFAULT_CONFIG.setdefault('VIDEO_EXCLUDE', [])
    DEFAULT_CONFIG.setdefault('VIDEO_EXCLUDEALL', False)
    DEFAULT_CONFIG.setdefault('VIDEO_SKIPTAG', '')

    DEFAULT_CONFIG['queue_convert'] = {}
    DEFAULT_CONFIG['created_galleries'] = {}
    DEFAULT_CONFIG['plugin_dir'] = os.path.dirname(os.path.realpath(__file__))

    if pelican:
        pelican.settings.setdefault('VIDEO_LIBRARY', p)
        pelican.settings.setdefault('VIDEO_GALLERY', (720, 400, 100))
        pelican.settings.setdefault('VIDEO_ARTICLE', (720, 400, 100))
        pelican.settings.setdefault('VIDEO_FORMAT', 'mp4')
        pelican.settings.setdefault('VIDEO_VCODEC', 'h264')
        pelican.settings.setdefault('VIDEO_ACODEC', 'aac')
        pelican.settings.setdefault('VIDEO_SAMPLERATE', 11025)
        pelican.settings.setdefault('VIDEO_CHANNELS', 2)
        pelican.settings.setdefault('VIDEO_CONVERT_JOBS', 1)
        # Need to change type, VIDEO_EXCLUDE must be iterable
        if pelican.settings['VIDEO_EXCLUDE'] is None:
            pelican.settings['VIDEO_EXCLUDE'] = []

        video_excludeall = pelican.settings['VIDEO_EXCLUDEALL'] == '1'
        pelican.settings['VIDEO_EXCLUDEALL'] = video_excludeall

        if pelican.settings['VIDEO_SKIPTAG'] is None:
            pelican.settings.setdefault('VIDEO_SKIPTAG', '')

def read_notes(filename, msg=None):
    notes = {}
    try:
        with pelican_open(filename) as text:
            for line in text.splitlines():
                if line.startswith('#'):
                    continue
                m = line.split(':', 1)
                if len(m) > 1:
                    pic = m[0].strip()
                    note = m[1].strip()
                    if pic and note:
                        notes[pic] = note
                else:
                    notes[line] = ''
    except Exception as e:
        if msg:
            logger.info('videos: {} at file {}'.format(msg, filename))

        debug_msg = 'videos: read_notes issue: {} at file {} {}'
        logger.debug(debug_msg.format(msg, filename, e))
    return notes

def build_license(license, author):
    year = datetime.datetime.now().year
    license_file = os.path.join(DEFAULT_CONFIG['plugin_dir'], 'licenses.json')

    with open(license_file) as data_file:
        licenses = json.load(data_file)

    if any(license in k for k in licenses):
        return licenses[license]['Text'].format(Author = author,
                                                Year = year,
                                                URL = licenses[license]['URL'])
    else:
        license_line = 'Copyright {Year} {Author}, All Rights Reserved'
        return license_line.format(Author = author, Year = year)

# Define a global lock as 'apply_async' doesn't support sharing primitives
GLock = multiprocessing.Lock()

def convert_worker(orig, converted, spec, settings):
    directory = os.path.split(converted)[0]

    GLock.acquire()
    if not os.path.exists(directory):
        try:
            os.makedirs(directory)
        except Exception:
            logger.exception('videos: could not create {}'.format(directory))
    else:
        if not settings['VIDEO_SKIPTAG']:
            debug_msg = 'videos: directory already exists at {}'
            logger.debug(debug_msg.format(os.path.split(converted)[0]))
    GLock.release()

    try:
        c = Converter() # FIXME: is Converter thread safe?
        conv = c.convert(orig, converted, spec)
        info_msg = '-> videos: converting ({:3d}%) | {}'
        for t in conv:
            print(info_msg.format(t, '*' * int(t/2)), end='\r')
        print(info_msg.format(100, '*' * int(t/2) + ' Done'), end='\n')

        if settings['VIDEO_SKIPTAG']:
            output_path = os.path.realpath(settings['OUTPUT_PATH'])
            output_videos_path = os.path.join(output_path, 'videos')
            cleaner_sh = os.path.join(output_videos_path, 'clean_skiptag.sh')
            converted_relpath = os.path.relpath(converted, output_videos_path)
            original_relpath = os.path.join(os.path.dirname(converted_relpath),
                                            os.path.basename(orig))
            try:
                GLock.acquire()
                with open(cleaner_sh, "a") as bash_script:
                    bash_line = "[ ! -f '{}' ] || rm -f '{}'"
                    print(bash_line.format(converted_relpath, original_relpath),
                          file=bash_script)
                GLock.release()
            except Exception as e:
                except_msg = 'videos: could not open file {}'
                logger.exception(except_msg.format(cleaner_sh))
                GLock.release()

    except Exception as e:
        error_msg = 'videos: could not convert {} {}'
        logger.exception(error_msg.format(orig, pprint.pformat(e)))

def sort_input_spec(input_spec):
    # H > W
    if input_spec[1] > input_spec[0]:
        return (input_spec[1], input_spec[0], input_spec[2])
    return input_spec 

def compute_output_spec(video_info, input_vspec, settings):
    # To make sure that original format is preserved, then regardless
    # of the original format, always sort what was requested.
    input_vspec = sort_input_spec(input_vspec)
    output_spec = {
        'format': settings['VIDEO_FORMAT'],
        'audio': {
            'codec': settings['VIDEO_ACODEC'],
            'samplerate': settings['VIDEO_SAMPLERATE'],
            'channels' : settings['VIDEO_CHANNELS']
        },
        'video': {
            'codec': settings['VIDEO_VCODEC'],
            'width': input_vspec[0],
            'height': input_vspec[1],
            'fps' : video_info[2]
        }
    }

    # portrait
    if video_info[1] > video_info[0]:
        output_spec['video']['width'] = input_vspec[1]
        output_spec['video']['height'] = input_vspec[0]

    # landscape or square
    return output_spec

def get_video_info(info):
    try:
        if info is None:
            logger.error('videos: no media info')
            return None

        # The API doesn't support more than 2 streams per file
        v_stream = info.streams[0]
        a_stream = info.streams[1]

        if v_stream is None or a_stream is None:
            logger.error('videos: no AV stream')
            return None

        return (v_stream.video_width, v_stream.video_height, v_stream.video_fps)

    except Exception as e:
        logger.exception('videos: none probed info {}'.format(pprint.pformat(e)))

def enqueue_convert(orig, converted, input_vspec, settings, do_enqueue):
    # Unfortunately need to compute width and height now, as templates
    # need to be aware of the actual spec being computed.
    try:
        c = Converter()
        info = c.probe(orig)
        video_info = get_video_info(info)
        output_spec = compute_output_spec(video_info, input_vspec, settings)
    except Exception as e:
        error_msg = 'videos: could not probe {} {}'
        logger.exception(error_msg.format(orig, pprint.pformat(e)))

    if converted not in DEFAULT_CONFIG['queue_convert']:
        if do_enqueue:
            DEFAULT_CONFIG['queue_convert'][converted] = (orig, output_spec)
        return (output_spec['video']['width'], output_spec['video']['height'])

    if DEFAULT_CONFIG['queue_convert'][converted] != (orig, output_spec):
        error_msg = 'videos: convert conflict for {}, {}-{} is not {}-{}'
        item_one = DEFAULT_CONFIG['queue_convert'][converted][0]
        item_two = DEFAULT_CONFIG['queue_convert'][converted][1]
        logger.error(error_msg.format(converted, item_one, item_two,
                                      orig, output_spec))

def convert_videos(generator, writer):
    if generator.settings['VIDEO_EXCLUDEALL']:
        logger.warning('videos: skip all galleries')
        return

    if generator.settings['VIDEO_CONVERT_JOBS'] == -1:
        generator.settings['VIDEO_CONVERT_JOBS'] = 1
        debug = True
    else:
        debug = False

    pool = multiprocessing.Pool(generator.settings['VIDEO_CONVERT_JOBS'])
    for converted, what in DEFAULT_CONFIG['queue_convert'].items():
        converted = os.path.join(generator.output_path, converted)
        abs_path = os.path.split(converted)[0]
        basename = os.path.basename(abs_path)

        if basename in generator.settings['VIDEO_EXCLUDE']:
            logger.warning('videos: skip gallery: {}'.format(basename))
            continue

        orig, spec = what
        if (not os.path.isfile(converted) or
            os.path.getmtime(orig) > os.path.getmtime(converted)):
            if debug:
                logger.debug('videos: synchronous convert: {}'.format(orig))
                convert_worker(orig, converted, spec, generator.settings)
            else:
                logger.debug('videos: asynchronous convert: {}'.format(orig))
                pool.apply_async(convert_worker,
                                 (orig, converted, spec, generator.settings))

    pool.close()
    pool.join()

def detect_content(content):
    hrefs = None

    def replacer(m):
        what = m.group('what')
        value = m.group('value')
        tag = m.group('tag')
        output = m.group(0)
 
        if what in ('video'):
            if value.startswith('/'):
                value = value[1:]

            path = os.path.join(os.path.expanduser(settings['VIDEO_LIBRARY']),
                                value)

            if os.path.isfile(path):
                original_filename = os.path.basename(path)
                do_enqueue = True
                do_rename = False
                if settings['VIDEO_SKIPTAG']:
                    if original_filename.startswith(settings['VIDEO_SKIPTAG']):
                        debug_msg = 'videos: flagged to skip: {}'
                        logger.debug(debug_msg.format(original_filename))
                        do_enqueue = False
                    else:
                        do_rename = True

                video_prefix = os.path.splitext(value)[0].lower()
                if do_rename:
                    video_prefix = settings['VIDEO_SKIPTAG'] + video_prefix

                if what == 'video':
                    ext = settings['VIDEO_FORMAT']
                    video_article = video_prefix + 'a.' + ext
                    enqueue_convert(path, os.path.join('videos', video_article),
                                    settings['VIDEO_ARTICLE'], settings, do_enqueue)

                    output = ''.join((
                        '<',
                        m.group('tag'),
                        m.group('attrs_before'),
                        m.group('src'),
                        '=',
                        m.group('quote'),
                        os.path.join(settings['SITEURL'], 'videos',
                                     video_article),
                        m.group('quote'),
                        m.group('attrs_after'),
                    ))
                else:
                    logger.error('videos: no video {}'.format(path))

        return output

    if hrefs is None:
        regex = r"""
            <\s*
            (?P<tag>[^\s\>]+)  # detect the tag
            (?P<attrs_before>[^\>]*)
            (?P<src>href|src)  # match tag with src and href attr
            \s*=
            (?P<quote>["\'])  # require value to be quoted
            (?P<path>{0}(?P<value>.*?))  # the url value
            (?P=quote)
            (?P<attrs_after>[^\>]*>)
        """.format(content.settings['INTRASITE_LINK_REGEX'])
        hrefs = re.compile(regex, re.X)

    if content._content and ('{video}' in content._content):
        settings = content.settings
        content._content = hrefs.sub(replacer, content._content)

def galleries_string_decompose(gallery_string):
    splitter_regex = re.compile(r'[\s,]*?({video}|{filename})')
    title_regex = re.compile(r'{(.+)}')
    galleries = map(unicode.strip if
                    sys.version_info.major == 2 else
                    str.strip,
                    filter(None, splitter_regex.split(gallery_string)))
    galleries = [gallery[1:] if
                 gallery.startswith('/') else
                 gallery for gallery in galleries]

    if len(galleries) % 2 == 0 and ' ' not in galleries:
        galleries = zip(zip(['type'] * len(galleries[0::2]),
                            galleries[0::2]),
                        zip(['location'] * len(galleries[0::2]),
                            galleries[1::2]))
        galleries = [dict(gallery) for gallery in galleries]

        for gallery in galleries:
            title = re.search(title_regex, gallery['location'])
            if title:
                gallery['title'] = title.group(1)
                gallery['location'] = re.sub(title_regex, '',
                                             gallery['location']).strip()
            else:
                gallery['title'] = DEFAULT_CONFIG['VIDEO_GALLERY_TITLE']
        return galleries
    else:
        error_msg = 'videos: unexpected gallery location format!\n{}'
        logger.error(error_msg.format(pprint.pformat(galleries)))

def process_gallery(generator, content, location):
    galleries = galleries_string_decompose(location)
    content.video_gallery = []

    for gallery in galleries:
        if gallery['location'] in DEFAULT_CONFIG['created_galleries']:
            item = DEFAULT_CONFIG['created_galleries'][gallery]
            content.video_gallery.append((gallery['location'], item))
            continue

        if gallery['type'] == '{video}':
            path = generator.settings['VIDEO_LIBRARY']
            dir_gallery = os.path.join(os.path.expanduser(path), gallery['location'])
            rel_gallery = gallery['location']
        elif gallery['type'] == '{filename}':
            base_path = os.path.join(generator.path, content.relative_dir)
            dir_gallery = os.path.join(base_path, gallery['location'])
            rel_gallery = os.path.join(content.relative_dir, gallery['location'])

        if os.path.isdir(dir_gallery):
            logger.info('videos: gallery detected: {}'.format(rel_gallery))
            dir_video = os.path.join('videos', rel_gallery.lower())

            try:
                captions = read_notes(os.path.join(dir_gallery, 'captions.txt'),
                                      msg='videos: no captions for gallery')
                blacklist = read_notes(os.path.join(dir_gallery, 'blacklist.txt'),
                                       msg='videos: no blacklist for gallery')
            except Exception as e:
                logger.debug('photos: exception {}'.format(pprint.pformat(e)))

            # HTML5 videos tag has several attribs. Here used only:
            # [({{ title }}, ({{ video }}, {{ width }}, {{ height }}, {{ format }}))]
            content_gallery = []
            tag_title = gallery['title']

            # Go over all the videos in dir_gallery
            for old_video in sorted(os.listdir(dir_gallery)):
                if old_video.startswith('.'):
                    continue
                if old_video.endswith('.txt'):
                    continue
                if old_video in blacklist:
                    continue

                do_enqueue = True
                do_rename = False
                if generator.settings['VIDEO_SKIPTAG']:
                    if old_video.startswith(generator.settings['VIDEO_SKIPTAG']):
                        debug_msg = 'videos: flagged to skip: {}'
                        logger.debug(debug_msg.format(old_video))
                        do_enqueue = False
                    else:
                        do_rename = True

                tag_format = generator.settings['VIDEO_FORMAT']
                # tag_name currently not passed to the template
                tag_name = os.path.splitext(old_video)[0].lower() + '.' + tag_format
                if do_rename:
                    tag_name = generator.settings['VIDEO_SKIPTAG'] + tag_name

                # relative path from /video/...
                tag_path = os.path.join(dir_video, tag_name)
                tag_spec = enqueue_convert(os.path.join(dir_gallery, old_video),
                                           tag_path,
                                           generator.settings['VIDEO_GALLERY'],
                                           generator.settings, do_enqueue)

                # Add more tag if needed, note: order matters!
                content_gallery.append((tag_path, tag_spec[0], tag_spec[1], tag_format))

            content.video_gallery.append((tag_title, content_gallery))
            debug_msg = 'videos: gallery data: {}'
            logger.debug(debug_msg.format(pprint.pformat(content.video_gallery)))
            DEFAULT_CONFIG['created_galleries']['gallery'] = content_gallery
        else:
            error_msg = 'videos: gallery does not exist: {} at {}'
            logger.error(error_msg.format(gallery['location'], dir_gallery))

def detect_gallery(generator, content):
    if 'video-gallery' in content.metadata:
        gallery = content.metadata.get('video-gallery')
        if gallery.startswith('{video}') or gallery.startswith('{filename}'):
            process_gallery(generator, content, gallery)
        elif gallery:
            error_msg = 'videos: gallery tag not recognized: {}'
            logger.error(error_msg.format(gallery))

def video_clipper(x):
    return x[8:] if x[8] == '/' else x[7:]

def file_clipper(x):
    return x[11:] if x[10] == '/' else x[10:]

def process_video(generator, content, video_input):
    if video_input.startswith('{video}'):
        path = generator.settings['VIDEO_LIBRARY']
        path = os.path.join(os.path.expanduser(path), video_clipper(video_input))
        video_name = video_clipper(video_input)
    elif video_input.startswith('{filename}'):
        path = os.path.join(generator.path, content.relative_dir,
                            file_clipper(video_input))
        video_name = file_clipper(video_input)

    if os.path.isfile(path):
        original_filename = os.path.basename(path)

        do_enqueue = True
        do_rename = False
        if generator.settings['VIDEO_SKIPTAG']:
            if original_filename.startswith(generator.settings['VIDEO_SKIPTAG']):
                debug_msg = 'videos: flagged to skip: {}'
                logger.debug(debug_msg.format(original_filename))
                do_enqueue = False
            else:
                do_rename = True

        ext = generator.settings['VIDEO_FORMAT']
        video = os.path.splitext(video_name)[0].lower() + 'a.' + ext
        if do_rename:
            video = generator.settings['VIDEO_SKIPTAG'] + video

        content.video_file = (os.path.basename(video_name).lower(),
                              os.path.join('videos', video))
        
        enqueue_convert(path, os.path.join('videos', video),
                        generator.settings['VIDEO_ARTICLE'],
                        settings, do_enqueue)
    else:
        error_msg = 'videos: no video for {} at {}'
        logger.error(error_msg.format(content.source_path, path))

def detect_video(generator, content):
    video = content.metadata.get('video', None)
    if video:
        if video.startswith('{video}') or video.startswith('{filename}'):
            process_video(generator, content, video)
        else:
            error_msg = 'videos: video tag not recognized: {}'
            logger.error(error_msg.format(video))

def detect_videos_and_galleries(generators):
    """Runs generator on both pages and articles."""
    for generator in generators:
        if isinstance(generator, ArticlesGenerator):
            for article in itertools.chain(generator.articles,
                                           generator.translations,
                                           generator.drafts):
                detect_video(generator, article)
                detect_gallery(generator, article)
        elif isinstance(generator, PagesGenerator):
            for page in itertools.chain(generator.pages,
                                        generator.translations,
                                        generator.hidden_pages):
                detect_video(generator, page)
                detect_gallery(generator, page)

def register():
    """Uses the new style of registration based on GitHub Pelican issue #314."""
    signals.initialized.connect(initialized)
    try:
        signals.content_object_init.connect(detect_content)
        signals.all_generators_finalized.connect(detect_videos_and_galleries)
        signals.article_writer_finalized.connect(convert_videos)
    except Exception as e:
        except_msg = 'videos: plugin failed to execute: {}'
        logger.exception(except_msg.format(pprint.pformat(e)))
